require_relative "condition_block"
require_relative "function"
require_relative "literal"

module Ascode
  module Parser
    class Main
      def initialize(code, impl_o = true)
        @function = Function.new
        @code = code

        @do_implicit_output = impl_o
      end

      def parse
        @ast = []
        @skip_chars = 0
        @implicit_output = @do_implicit_output

        @code.split("").to_enum.each_with_index do |char, index|
          if @skip_chars > 0
            @skip_chars -= 1
            next
          end

          character char, index
        end

        @ast.push(type: :io, action: "output") if @implicit_output

        @ast
      end

      def character(char, index)
        if char == "\"" || char =~ /[0-9.]/
          literal index
        elsif char == "["
          block index
        else
          function char
        end
      end

      def literal(index)
        literal = Literal.new @code, index
        @ast.push literal.parse
        @skip_chars = literal.length
      end

      def block(index)
        block = ConditionBlock.new @code, index
        @ast.push block.parse
        @skip_chars = block.block.length
      end

      def function(char)
        function = @function.parse char
        return unless function
        @implicit_output = false if function[:action] == "output"
        @ast.push function
      end
    end
  end
end
